"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Channel = void 0;
const presence_channel_1 = require("./presence-channel");
const private_channel_1 = require("./private-channel");
const log_1 = require("./../log");
class Channel {
    constructor(io, options) {
        this.io = io;
        this.options = options;
        this._privateChannels = ['private-*', 'presence-*'];
        this._clientEvents = ['client-*'];
        this.private = new private_channel_1.PrivateChannel(options);
        this.presence = new presence_channel_1.PresenceChannel(io, options);
        if (this.options.devMode) {
            log_1.Log.success('Channels are ready.');
        }
    }
    join(socket, data) {
        if (data.channel) {
            if (this.isPrivate(data.channel)) {
                this.joinPrivate(socket, data);
            }
            else {
                socket.join(data.channel);
                this.onJoin(socket, data.channel);
            }
        }
    }
    clientEvent(socket, data) {
        var _a;
        try {
            data = JSON.parse(data);
        }
        catch (e) {
            data = data;
        }
        if (data.event && data.channel) {
            console.log("Client event: " + data.event);
            console.log("Client channel: " + data.channel);
            if (this.isClientEvent(data.event) &&
                this.isPrivate(data.channel) &&
                this.isInChannel(socket, data.channel)) {
                const whisperEventName = `.client-${data.event.replace('client-', '')}`;
                console.log("Whisper event name: " + whisperEventName);
                (_a = this.io.sockets.sockets.get(socket.id)) === null || _a === void 0 ? void 0 : _a.to(data.channel).emit(whisperEventName, data.data);
            }
        }
    }
    leave(socket, channel, reason) {
        if (channel) {
            if (this.isPresence(channel)) {
                this.presence.leave(socket, channel);
            }
            socket.leave(channel);
            if (this.options.devMode) {
                log_1.Log.info(`[${new Date().toISOString()}] - ${socket.id} left channel: ${channel} (${reason})`);
            }
        }
    }
    isPrivate(channel) {
        let isPrivate = false;
        this._privateChannels.forEach(privateChannel => {
            let regex = new RegExp(privateChannel.replace('\*', '.*'));
            if (regex.test(channel))
                isPrivate = true;
        });
        return isPrivate;
    }
    joinPrivate(socket, data) {
        this.private.authenticate(socket, data).then(res => {
            socket.join(data.channel);
            if (this.isPresence(data.channel)) {
                var member = res.channel_data;
                try {
                    member = JSON.parse(res.channel_data);
                }
                catch (e) { }
                this.presence.join(socket, data.channel, member);
            }
            this.onJoin(socket, data.channel);
        }, error => {
            if (this.options.devMode) {
                log_1.Log.error(error.reason);
            }
            this.io.to(socket.id)
                .emit('subscription_error', data.channel, error.status);
        });
    }
    isPresence(channel) {
        return channel.lastIndexOf('presence-', 0) === 0;
    }
    onJoin(socket, channel) {
        if (this.options.devMode) {
            log_1.Log.info(`[${new Date().toISOString()}] - ${socket.id} joined channel: ${channel}`);
        }
    }
    isClientEvent(event) {
        let isClientEvent = false;
        this._clientEvents.forEach(clientEvent => {
            let regex = new RegExp(clientEvent.replace('\*', '.*'));
            if (regex.test(event))
                isClientEvent = true;
        });
        return isClientEvent;
    }
    isInChannel(socket, channel) {
        return socket.rooms.has(channel);
    }
}
exports.Channel = Channel;
