"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EchoNodeServer = void 0;
const subscribers_1 = require("./subscribers");
const channels_1 = require("./channels");
const server_1 = require("./server");
const api_1 = require("./api");
const log_1 = require("./log");
const packageFile = require('../package.json');
const crypto_1 = require("crypto");
class EchoNodeServer {
    constructor() {
        this.defaultOptions = {
            authHost: 'http://localhost',
            authEndpoint: '/broadcasting/auth',
            allowSubDomains: false,
            multiTenant: false,
            clients: [],
            database: 'redis',
            databaseConfig: {
                redis: {},
                sqlite: {
                    databasePath: '/database/laravel-echo-server.sqlite'
                }
            },
            devMode: false,
            host: null,
            port: 6001,
            maxConcurrentAuthRequests: null,
            protocol: "http",
            socketio: {},
            secureOptions: crypto_1.constants.SSL_OP_NO_TLSv1,
            sslCertPath: '',
            sslKeyPath: '',
            sslCertChainPath: '',
            sslPassphrase: '',
            allowSelfSigned: false,
            subscribers: {
                http: true,
                redis: true
            },
            apiOriginAllow: {
                allowCors: false,
                allowOrigin: '',
                allowMethods: '',
                allowHeaders: ''
            }
        };
    }
    run(options) {
        return new Promise((resolve, reject) => {
            this.options = Object.assign(this.defaultOptions, options);
            this.startup();
            this.server = new server_1.Server(this.options);
            this.server.init().then(io => {
                this.init(io).then(() => {
                    log_1.Log.info('\nServer ready!\n');
                    resolve(this);
                }, error => log_1.Log.error(error));
            }, error => log_1.Log.error(error));
        });
    }
    init(io) {
        return new Promise((resolve, reject) => {
            var _a, _b, _c;
            this.channel = new channels_1.Channel(io, this.options);
            this.subscribers = [];
            if (this.options.subscribers.http)
                this.subscribers.push(new subscribers_1.HttpSubscriber((_a = this.server) === null || _a === void 0 ? void 0 : _a.express, this.options, (_b = this.server) === null || _b === void 0 ? void 0 : _b.io));
            if (this.options.subscribers.redis)
                this.subscribers.push(new subscribers_1.RedisSubscriber(this.options));
            this.httpApi = new api_1.HttpApi(io, this.channel, (_c = this.server) === null || _c === void 0 ? void 0 : _c.express, this.options.apiOriginAllow);
            this.httpApi.init();
            this.onConnect();
            this.listen().then(() => resolve(), err => log_1.Log.error(err));
        });
    }
    startup() {
        log_1.Log.title(`\nL A R A V E L  E C H O  N O D E  S E R V E R\n`);
        log_1.Log.info(`version ${packageFile.version}\n`);
        if (this.options.multiTenant) {
            log_1.Log.info('Mutitenant enabled.');
        }
        if (this.options.allowSubDomains) {
            log_1.Log.info('SubDomains enabled.');
        }
        if (this.options.devMode) {
            log_1.Log.warning('Starting server in DEV mode...\n');
        }
        else {
            log_1.Log.info('Starting server...\n');
        }
    }
    stop() {
        var _a, _b;
        let promises = [];
        (_a = this.subscribers) === null || _a === void 0 ? void 0 : _a.forEach(subscriber => {
            promises.push(subscriber.unsubscribe());
        });
        promises.push((_b = this.server) === null || _b === void 0 ? void 0 : _b.io.close());
        return Promise.all(promises).then(() => {
            this.subscribers = [];
        });
    }
    listen() {
        return new Promise((resolve, reject) => {
            var _a;
            let subscribePromises = ((_a = this.subscribers) === null || _a === void 0 ? void 0 : _a.map(subscriber => {
                return subscriber.subscribe((channel, message) => {
                    return this.broadcast(channel, message);
                });
            })) || [];
            Promise.all(subscribePromises).then(() => resolve(), reject);
        });
    }
    find(socket_id) {
        var _a;
        return (_a = this.server) === null || _a === void 0 ? void 0 : _a.io.sockets.connected[socket_id];
    }
    broadcast(channel, message) {
        if (message.socket && this.find(message.socket)) {
            return this.toOthers(this.find(message.socket), channel, message);
        }
        else {
            return this.toAll(channel, message);
        }
    }
    toOthers(socket, channel, message) {
        socket.broadcast.to(channel)
            .emit(message.event, channel, message.data);
        return true;
    }
    toAll(channel, message) {
        var _a;
        (_a = this.server) === null || _a === void 0 ? void 0 : _a.io.to(channel).emit(message.event, channel, message.data);
        return true;
    }
    onConnect() {
        var _a;
        (_a = this.server) === null || _a === void 0 ? void 0 : _a.io.on('connection', (socket) => {
            this.onSubscribe(socket);
            this.onUnsubscribe(socket);
            this.onDisconnecting(socket);
            this.onClientEvent(socket);
        });
    }
    onTyping(socket) {
        socket.on('typing', (data) => {
            var _a;
            const { channel } = data;
            if (!channel) {
                log_1.Log.error(`Typing event missing channel information.`);
                return;
            }
            log_1.Log.info("Typing event data received:" + data);
            (_a = this.server) === null || _a === void 0 ? void 0 : _a.io.to(channel).emit('client-typing', Object.assign({ user: socket.id }, data));
            log_1.Log.info(`User ${socket.id} is typing in channel: ${channel}`);
        });
    }
    onStopTyping(socket) {
        socket.on('stop_typing', (data) => {
            var _a;
            const { channel } = data;
            console.log("StopTyping event channel received: " + channel);
            if (!channel) {
                log_1.Log.error(`StopTyping event missing channel information.`);
                return;
            }
            log_1.Log.info("StopTyping event data received: " + data);
            (_a = this.server) === null || _a === void 0 ? void 0 : _a.io.to(channel).emit('stop_typing', Object.assign({ user: socket.id }, data));
            log_1.Log.info(`User ${socket.id} stopped typing in channel: ${channel}`);
        });
    }
    onSubscribe(socket) {
        socket.on('subscribe', (data) => {
            var _a;
            (_a = this.channel) === null || _a === void 0 ? void 0 : _a.join(socket, data);
        });
    }
    onUnsubscribe(socket) {
        socket.on('unsubscribe', (data) => {
            var _a;
            (_a = this.channel) === null || _a === void 0 ? void 0 : _a.leave(socket, data.channel, 'unsubscribed');
        });
    }
    onDisconnecting(socket) {
        socket.on('disconnecting', (reason) => {
            Object.keys(socket.rooms).forEach(room => {
                var _a;
                if (room !== socket.id) {
                    (_a = this.channel) === null || _a === void 0 ? void 0 : _a.leave(socket, room, reason);
                }
            });
        });
    }
    onClientEvent(socket) {
        socket.on('client event', (data) => {
            var _a;
            const { channel, event } = data;
            console.log("event: " + event);
            if (!event || !channel) {
                log_1.Log.error(`Event or channel is missing in the client event.`);
                return;
            }
            log_1.Log.info(`Relaying general client event: ${event}`);
            (_a = this.channel) === null || _a === void 0 ? void 0 : _a.clientEvent(socket, data);
        });
    }
}
exports.EchoNodeServer = EchoNodeServer;
